% CLASS OrthMesh, meshing d-hypercubes with simplices or hypercubes.
%
% SEE
%   OrthMesh/OrthMesh : constructor
%   OrthMesh/plot     : plot function
%   OrthMesh/disp     : display information on an OrthMesh object
%
%    Copyright (C) 2016-2020 F. Cuvelier 
classdef OrthMesh  %< handle 

  properties %(SetAccess = protected)
    d = 0; % space dimension
    order=1;
    Mesh=[]; % main mesh (EltMesh)
    Faces={}; % cells of m-face meshes
    type=[]; % 'simplex' or 'orthotope'
    box=[]; 
  end
  
  methods
    function self = OrthMesh(varargin)
    % OrthMesh constructor
    %   
    % USAGE
    %   Oh=OrthMesh(N)
    %   Oh=OrthMesh(d,N)
    %   Oh=OrthMesh(N, <key>, <value>, ...)
    %   Oh=OrthMesh(d,N, <key>, <value>, ...)
    %
    %   Parameter d is the space dimension and parameter N the number of
    %   discretization in each direction. 
    %   if d is not given then N is a 1-by-d array otherwise N could also be
    %   a scalar.
    %   The d,N parameters can be followed by key/value pairs to specify
    %   options. The keys will be the strings:
    %     'type' to specify the kind of mesh with value equal to 'orthotope' or
    %           'simplex' (default).
    %     'order' order of the mesh element (default 1)
    %     'box' to describe the orthotope to mesh. The default orthotope is
    %           the unit d-hypercube [0,1]^d. The associated value is 
    %           the d-by-2 array : [zeros(d,1),ones(d,1)]
    %     'm_min' used to only build the m-Faces for m in {m_min,...,d-1}.
    %           Default value is 0.
    %     'mapping' used to apply on the mesh a mapping function given by a 
    %           function handle.
    %
    % SAMPLES
    %   Oh=OrthMesh(3,10);
    %   Oh=OrthMesh([5,7],'type','orthotope');
    %   Oh=OrthMesh(2,[5,7],'type','simplex','order',3);
    %   Oh=OrthMesh([5,7,6],'box',[-2,2;-1,1;0,1]);
    %
    %    Copyright (C) 2016-2020 F. Cuvelier 
      idx=find(cellfun(@ischar,varargin));
      if isempty(idx)
        assert( ismember(nargin,[1,2]) )
        if nargin==2
          d=varargin{1};
          N=varargin{2};
          if size(N,2)==1,N=N*ones(1,d);end
          assert(size(N,2)==d)
        else
          N=varargin{1};
          d=size(N,2);
        end
        varargin={};
      else
        assert(ismember(idx(1),[2,3]))
        if (idx(1)==3)
          d=varargin{1};
          N=varargin{2};
          if size(N,2)==1,N=N*ones(1,d);end
          assert(size(N,2)==d)
        else  
          N=varargin{1};
          d=size(N,2);
        end
        varargin={varargin{idx(1):end}};
      end  

      p = inputParser;
      p.addParamValue('type', 'simplex', @(x) ismember(x,{'simplex','orthotope'}) );
      p.addParamValue('box', [zeros(d,1),ones(d,1)], @(x) (sum(size(x)==[d,2])==2) );
      p.addParamValue('m_min', 0, @(x) (x<=d) && (x>=0) );
      p.addParamValue('order', 1 );
      p.addParamValue('mapping', []);
      p.parse(varargin{:});
      R=p.Results;
      self.box=R.box;
      self.type=R.type;
      self.order=R.order;
      if strcmp(R.type,'simplex')
        funMesh=@(N) fc_hypermesh.CG.TessSim(N,R.order);
        funFaces=@(N,m) fc_hypermesh.CG.genLevelFaces(N,m,'type','simplex','order',R.order);
      else
        funMesh=@(N) fc_hypermesh.CG.TessHyp(N,R.order);
        funFaces=@(N,m) fc_hypermesh.CG.genLevelFaces(N,m,'type','orthotope','order',R.order);
      end
      self.d=d;
      mappingfun=@(q) diag(1./(N))*q;% mapping to unit hypercube
      mappingfun=@(q) fc_hypermesh.MappingBox(mappingfun(q),R.box);
      [q,me]=funMesh(N);
      if ~isempty(R.mapping)
        mappingfun=@(q) R.mapping(mappingfun(q));
      end
      labName=fc_hypermesh.funLabName(d,d);
      self.Mesh=fc_hypermesh.EltMesh(d,mappingfun(q),me,'label',labName(1),'order',R.order,'type',R.type);
      k=1;
      for m=d-1:-1:R.m_min
        labName=fc_hypermesh.funLabName(d,m);
        sTh=funFaces(N,m);
        nsTh=length(sTh);
        colors = fc_tools.graphics.selectColors(nsTh);
        for j=1:length(sTh)
          sTh(j).set('color',colors(j,:),'label',labName(j),'mapping',mappingfun);
          % Use set property due to 'BUG' in Octave 4.0.x not in 4.4.x
          % Following is OK with Matlab and Octave 4.4.x
          %sTh(j).q=mappingfun(sTh(j).q);
          %sTh(j).color=colors(j,:);
          %sTh(j).label=labName(j);
        end
        self.Faces{k}=sTh;
        k=k+1;
      end
    end
    
    function disp(self)
      fprintf('Mesh type : %s\n',self.type);
      fprintf('dimension : %d\n',self.d);
      fprintf('order     : %d\n',self.order);
      fprintf('nq        : %d\n',self.Mesh.nq);
      fprintf('nme       : %d\n',self.Mesh.nme);
      for i=1:length(self.Faces)
        fprintf('Number of %d-faces : %d\n',self.Faces{i}(1).m,numel(self.Faces{i}))  
      end
    end % disp method
    
    function mem=get_memory(self) % in bytes
    % return memory used by all arrays in the OrthMesh object (in bytes)
      mem=self.Mesh.get_memory();
      for i=1:length(self.Faces)
        for k=1:length(self.Faces{i})
          mem=mem+self.Faces{i}(k).get_memory();
        end
      end
    end
    
    function idx=getFacesIndex(self,m)
      idx=find( [self.d-1:-1:0] == m );
    end % getFacesIndex method
    
    function labName=funLabName(self,m)
      if self.d==m
        labName=@(i) ['\Omega_{',num2str(i),'}'];
        return;
      elseif self.d-1==m
        labName=@(i) ['\Gamma_{',num2str(i),'}'];
        return;
      elseif self.d-2==m
        labName=@(i) ['\partial\Gamma_{',num2str(i),'}'];
        return
      else
        p=self.d-m-1;
        labName=@(i) ['\partial^{',num2str(p),'}\Gamma_{',num2str(i),'}'];
      end
    end % funLabName method
    
    function varargout=plotmesh(self,varargin)
      [h,cLegend]=fc_hypermesh.OrthMesh_fun.plotmesh(self,varargin{:});
      if nargout>=1,varargout{1}=h;end
      if nargout==2,varargout{2}=cLegend;end
    end % plotmesh method   
    
    
    function plotnodes(self,varargin)
      fc_hypermesh.OrthMesh_fun.plotnodes(self, varargin{:});
    end % plotnodes method
    
    function plotnodesNumber(self,varargin)
      fc_hypermesh.OrthMesh_fun.plotnodesNumber(self, varargin{:});
    end % plotnodesNumber method
    
    function ploteltsNumber(self,varargin)
      fc_hypermesh.OrthMesh_fun.ploteltsNumber(self, varargin{:});
    end % plotEltsNumber method
    
  end %methods
end %classdef
